@@ -1,6 +1,15 @@
 #include "curl-common.h"
 #include "callbacks.h"
 
+/* These are defined in typechecking.c */
+extern int r_curl_is_slist_option(CURLoption x);
+extern int r_curl_is_long_option(CURLoption x);
+extern int r_curl_is_off_t_option(CURLoption x);
+extern int r_curl_is_string_option(CURLoption x);
+extern int r_curl_is_postfields_option(CURLoption x);
+
+#define make_string(x) x ? Rf_mkString(x) : ScalarString(NA_STRING)
+
 #ifndef MAX_PATH
 #define MAX_PATH 1024
 #endif
@@ -17,9 +26,8 @@
 #define HAS_CURLOPT_EXPECT_100_TIMEOUT_MS 1
 #endif
 
-
 char CA_BUNDLE[MAX_PATH];
-static struct curl_slist * default_headers;
+extern int windows_openssl;
 
 SEXP R_set_bundle(SEXP path){
   strcpy(CA_BUNDLE, CHAR(asChar(path)));
@@ -36,6 +44,8 @@ void clean_handle(reference *ref){
   if(ref->refCount == 0){
     if(ref->headers)
       curl_slist_free_all(ref->headers);
+    if(ref->custom)
+      curl_slist_free_all(ref->custom);
     if(ref->form)
       curl_formfree(ref->form);
     if(ref->handle)
@@ -65,8 +75,54 @@ size_t dummy_read(char *buffer, size_t size, size_t nitems, void *instream){
   return 0;
 }
 
+#ifdef HAS_XFERINFOFUNCTION
+#define xftype curl_off_t
+#else
+#define xftype double
+#endif
+
+static int xferinfo_callback(void *clientp, xftype dltotal, xftype dlnow, xftype ultotal, xftype ulnow){
+  static xftype dlprev = 0;
+  static xftype ulprev = 0;
+  if(dlnow && dlnow != dlprev){
+    dlprev = dlnow;
+    if(dltotal){
+      int pct_dn = (100 * dlnow)/dltotal;
+      REprintf("\r [%d%%] Downloaded %.0lf bytes...", (double) dlnow, pct_dn);
+      if(dlnow == dltotal)
+        REprintf("\n");
+    } else {
+      REprintf("\r Downloaded %.0lf bytes...", (double) dlnow);
+    }
+  } else if(ulnow && ulnow != ulprev){
+    ulprev = ulnow;
+    int pct_up = (100 * ulnow)/ultotal;
+    REprintf("\r [%d%%] Uploaded %.0lf bytes...", (double) ulnow, pct_up);
+    if(ulnow == ultotal)
+      REprintf("\n");
+  }
+  return 0;
+}
+
+/* Set default headers here, these are only allocated once */
+static struct curl_slist * default_headers(){
+  static struct curl_slist * headers = NULL;
+  if(headers == NULL){
+    headers = curl_slist_append(headers, "Expect:");
+  }
+  return headers;
+}
+
+static void set_headers(reference *ref, struct curl_slist *newheaders){
+  if(ref->headers)
+    curl_slist_free_all(ref->headers);
+  ref->headers = newheaders;
+  assert(curl_easy_setopt(ref->handle, CURLOPT_HTTPHEADER,
+                          newheaders ? newheaders : default_headers()));
+}
+
 /* These are defaulst that we always want to set */
-void set_handle_defaults(reference *ref){
+static void set_handle_defaults(reference *ref){
 
   /* the actual curl handle */
   CURL *handle = ref->handle;
@@ -78,18 +134,20 @@ void set_handle_defaults(reference *ref){
   curl_easy_setopt(handle, CURLOPT_HEADERDATA, &(ref->resheaders));
 
   #ifdef _WIN32
-  if(CA_BUNDLE != NULL && strlen(CA_BUNDLE)){
-    /* on windows a cert bundle is included with R version 3.2.0 */
-    curl_easy_setopt(handle, CURLOPT_CAINFO, CA_BUNDLE);
-  } else {
-    /* disable cert validation for older versions of R */
-    curl_easy_setopt(handle, CURLOPT_SSL_VERIFYHOST, 0L);
-    curl_easy_setopt(handle, CURLOPT_SSL_VERIFYPEER, 0L);
+  if(windows_openssl == 1){
+    if( CA_BUNDLE != NULL && strlen(CA_BUNDLE)){
+      /* on windows a cert bundle is included with R version 3.2.0 */
+      curl_easy_setopt(handle, CURLOPT_CAINFO, CA_BUNDLE);
+    } else {
+      /* disable cert validation for older versions of R */
+      curl_easy_setopt(handle, CURLOPT_SSL_VERIFYHOST, 0L);
+      curl_easy_setopt(handle, CURLOPT_SSL_VERIFYPEER, 0L);
+    }
   }
   #endif
 
   /* needed to support compressed responses */
-  assert(curl_easy_setopt(handle, CURLOPT_ENCODING, "gzip, deflate"));
+  assert(curl_easy_setopt(handle, CURLOPT_ENCODING, ""));
 
   /* follow redirect */
   assert(curl_easy_setopt(handle, CURLOPT_FOLLOWLOCATION, 1L));
@@ -132,11 +190,18 @@ void set_handle_defaults(reference *ref){
   curl_easy_setopt(handle, CURLOPT_SSL_OPTIONS, CURLSSLOPT_NO_REVOKE);
 #endif
 
-  /* set default headers (disables the Expect: http 100)*/
+  /* set default progress printer (disabled by default) */
+#ifdef HAS_XFERINFOFUNCTION
+  assert(curl_easy_setopt(handle, CURLOPT_XFERINFOFUNCTION, xferinfo_callback));
+#else
+  assert(curl_easy_setopt(handle, CURLOPT_PROGRESSFUNCTION, xferinfo_callback));
+#endif
+
+  /* Disable the 'Expect: 100' header (deprecated in recent libcurl) */
+  set_headers(ref, NULL);
 #ifdef HAS_CURLOPT_EXPECT_100_TIMEOUT_MS
   assert(curl_easy_setopt(handle, CURLOPT_EXPECT_100_TIMEOUT_MS, 0L));
 #endif
-  assert(curl_easy_setopt(handle, CURLOPT_HTTPHEADER, default_headers));
 }
 
 SEXP R_new_handle(){
@@ -145,10 +210,11 @@ SEXP R_new_handle(){
   ref->handle = curl_easy_init();
   total_handles++;
   set_handle_defaults(ref);
-  SEXP ptr = PROTECT(R_MakeExternalPtr(ref, R_NilValue, R_NilValue));
+  SEXP prot = PROTECT(allocVector(VECSXP, 5)); //for protecting callback functions
+  SEXP ptr = PROTECT(R_MakeExternalPtr(ref, R_NilValue, prot));
   R_RegisterCFinalizerEx(ptr, fin_handle, TRUE);
   setAttrib(ptr, R_ClassSymbol, mkString("curl_handle"));
-  UNPROTECT(1);
+  UNPROTECT(2);
   ref->handleptr = ptr;
   return ptr;
 }
@@ -157,28 +223,41 @@ SEXP R_handle_reset(SEXP ptr){
   //reset all fields
   reference *ref = get_ref(ptr);
   set_form(ref, NULL);
-  set_headers(ref, NULL);
   reset_errbuf(ref);
   curl_easy_reset(ref->handle);
 
+  //clear custom vector field
+  if(ref->custom){
+    curl_slist_free_all(ref->custom);
+    ref->custom = NULL;
+  }
+
   //restore default settings
   set_handle_defaults(ref);
   return ScalarLogical(1);
 }
 
-int opt_is_linked_list(int key) {
-  // These four options need linked lists of various forms - determined
-  // from inspection of curl.h
-  return
-    key == 10023 || // CURLOPT_HTTPHEADER
-    key == 10024 || // CURLOPT_HTTPPOST
-    key == 10070 || // CURLOPT_TELNETOPTIONS
-    key == 10104 || // CURLOPT_HTTP200ALIASES
-    key == 10228;   // CURLOPT_PROXYHEADER
+SEXP R_handle_setheaders(SEXP ptr, SEXP vec){
+  if(!isString(vec))
+    error("header vector must be a string.");
+  set_headers(get_ref(ptr), vec_to_slist(vec));
+  return ScalarLogical(1);
+}
+
+SEXP R_handle_getheaders(SEXP ptr){
+  reference *ref = get_ref(ptr);
+  return slist_to_vec(ref->headers);
+}
+
+SEXP R_handle_getcustom(SEXP ptr){
+  reference *ref = get_ref(ptr);
+  return slist_to_vec(ref->custom);
 }
 
 SEXP R_handle_setopt(SEXP ptr, SEXP keys, SEXP values){
+  reference *ref = get_ref(ptr);
   CURL *handle = get_handle(ptr);
+  SEXP prot = R_ExternalPtrProtected(ptr);
   SEXP optnames = PROTECT(getAttrib(values, R_NamesSymbol));
 
   if(!isInteger(keys))
@@ -202,6 +281,7 @@ SEXP R_handle_setopt(SEXP ptr, SEXP keys, SEXP values){
                               (curl_progress_callback) R_curl_callback_xferinfo));
       assert(curl_easy_setopt(handle, CURLOPT_XFERINFODATA, val));
       assert(curl_easy_setopt(handle, CURLOPT_NOPROGRESS, 0));
+      SET_VECTOR_ELT(prot, 1, val); //protect gc
 #endif
     } else if (key == CURLOPT_PROGRESSFUNCTION) {
       if (TYPEOF(val) != CLOSXP)
@@ -211,6 +291,7 @@ SEXP R_handle_setopt(SEXP ptr, SEXP keys, SEXP values){
         (curl_progress_callback) R_curl_callback_progress));
       assert(curl_easy_setopt(handle, CURLOPT_PROGRESSDATA, val));
       assert(curl_easy_setopt(handle, CURLOPT_NOPROGRESS, 0));
+      SET_VECTOR_ELT(prot, 2, val); //protect gc
     } else if (key == CURLOPT_READFUNCTION) {
       if (TYPEOF(val) != CLOSXP)
         error("Value for option %s (%d) must be a function.", optname, key);
@@ -218,6 +299,7 @@ SEXP R_handle_setopt(SEXP ptr, SEXP keys, SEXP values){
       assert(curl_easy_setopt(handle, CURLOPT_READFUNCTION,
         (curl_read_callback) R_curl_callback_read));
       assert(curl_easy_setopt(handle, CURLOPT_READDATA, val));
+      SET_VECTOR_ELT(prot, 3, val); //protect gc
     } else if (key == CURLOPT_DEBUGFUNCTION) {
       if (TYPEOF(val) != CLOSXP)
         error("Value for option %s (%d) must be a function.", optname, key);
@@ -225,18 +307,29 @@ SEXP R_handle_setopt(SEXP ptr, SEXP keys, SEXP values){
       assert(curl_easy_setopt(handle, CURLOPT_DEBUGFUNCTION,
         (curl_debug_callback) R_curl_callback_debug));
       assert(curl_easy_setopt(handle, CURLOPT_DEBUGDATA, val));
+      SET_VECTOR_ELT(prot, 4, val); //protect gc
     } else if (key == CURLOPT_URL) {
       /* always use utf-8 for urls */
       const char * url_utf8 = translateCharUTF8(STRING_ELT(val, 0));
       assert(curl_easy_setopt(handle, CURLOPT_URL, url_utf8));
-    } else if (opt_is_linked_list(key)) {
-      error("Option %s (%d) not supported.", optname, key);
-    } else if(key < 10000){
+    } else if(key == CURLOPT_HTTPHEADER){
+      R_handle_setheaders(ptr, val);
+    } else if (r_curl_is_slist_option(key)) {
+      if(!isString(val))
+        error("Value for option %s (%d) must be a string vector", optname, key);
+      ref->custom = vec_to_slist(val);
+      assert(curl_easy_setopt(handle, key, ref->custom));
+    } else if(r_curl_is_long_option(key)){
       if(!isNumeric(val) || length(val) != 1) {
         error("Value for option %s (%d) must be a number.", optname, key);
       }
       assert(curl_easy_setopt(handle, key, (long) asInteger(val)));
-    } else if(key < 20000){
+    } else if(r_curl_is_off_t_option(key)){
+      if(!isNumeric(val) || length(val) != 1) {
+        error("Value for option %s (%d) must be a number.", optname, key);
+      }
+      assert(curl_easy_setopt(handle, key, (curl_off_t) asReal(val)));
+    } else if(r_curl_is_string_option(key) || r_curl_is_postfields_option(key)){
       switch (TYPEOF(val)) {
       case RAWSXP:
         if(key == CURLOPT_POSTFIELDS || key == CURLOPT_COPYPOSTFIELDS)
@@ -251,11 +344,6 @@ SEXP R_handle_setopt(SEXP ptr, SEXP keys, SEXP values){
       default:
         error("Value for option %s (%d) must be a string or raw vector.", optname, key);
       }
-    } else if(key >= 30000 && key < 40000){
-      if(!isNumeric(val) || length(val) != 1) {
-        error("Value for option %s (%d) must be a number.", optname, key);
-      }
-      assert(curl_easy_setopt(handle, key, (curl_off_t) asReal(val)));
     } else {
       error("Option %s (%d) not supported.", optname, key);
     }
@@ -264,13 +352,6 @@ SEXP R_handle_setopt(SEXP ptr, SEXP keys, SEXP values){
   return ScalarLogical(1);
 }
 
-SEXP R_handle_setheaders(SEXP ptr, SEXP vec){
-  if(!isString(vec))
-    error("header vector must be a string.");
-  set_headers(get_ref(ptr), vec_to_slist(vec));
-  return ScalarLogical(1);
-}
-
 SEXP R_handle_setform(SEXP ptr, SEXP form){
   if(!isVector(form))
     error("Form must be a list.");
@@ -323,6 +404,12 @@ SEXP make_status(CURL *handle){
   return ScalarInteger(res_status);
 }
 
+SEXP make_ctype(CURL *handle){
+  char * ct;
+  assert(curl_easy_getinfo(handle, CURLINFO_CONTENT_TYPE, &ct));
+  return make_string(ct);
+}
+
 SEXP make_url(CURL *handle){
   char *res_url;
   assert(curl_easy_getinfo(handle, CURLINFO_EFFECTIVE_URL, &res_url));
@@ -355,13 +442,14 @@ SEXP make_rawvec(unsigned char *ptr, size_t size){
 }
 
 SEXP make_namesvec(){
-  SEXP names = PROTECT(allocVector(STRSXP, 6));
+  SEXP names = PROTECT(allocVector(STRSXP, 7));
   SET_STRING_ELT(names, 0, mkChar("url"));
   SET_STRING_ELT(names, 1, mkChar("status_code"));
-  SET_STRING_ELT(names, 2, mkChar("headers"));
-  SET_STRING_ELT(names, 3, mkChar("modified"));
-  SET_STRING_ELT(names, 4, mkChar("times"));
-  SET_STRING_ELT(names, 5, mkChar("content"));
+  SET_STRING_ELT(names, 2, mkChar("type"));
+  SET_STRING_ELT(names, 3, mkChar("headers"));
+  SET_STRING_ELT(names, 4, mkChar("modified"));
+  SET_STRING_ELT(names, 5, mkChar("times"));
+  SET_STRING_ELT(names, 6, mkChar("content"));
   UNPROTECT(1);
   return names;
 }
@@ -372,13 +460,14 @@ SEXP R_get_handle_cookies(SEXP ptr){
 
 SEXP make_handle_response(reference *ref){
   CURL *handle = ref->handle;
-  SEXP res = PROTECT(allocVector(VECSXP, 6));
+  SEXP res = PROTECT(allocVector(VECSXP, 7));
   SET_VECTOR_ELT(res, 0, make_url(handle));
   SET_VECTOR_ELT(res, 1, make_status(handle));
-  SET_VECTOR_ELT(res, 2, make_rawvec(ref->resheaders.buf, ref->resheaders.size));
-  SET_VECTOR_ELT(res, 3, make_filetime(handle));
-  SET_VECTOR_ELT(res, 4, make_timevec(handle));
-  SET_VECTOR_ELT(res, 5, R_NilValue);
+  SET_VECTOR_ELT(res, 2, make_ctype(handle));
+  SET_VECTOR_ELT(res, 3, make_rawvec(ref->resheaders.buf, ref->resheaders.size));
+  SET_VECTOR_ELT(res, 4, make_filetime(handle));
+  SET_VECTOR_ELT(res, 5, make_timevec(handle));
+  SET_VECTOR_ELT(res, 6, R_NilValue);
   setAttrib(res, R_NamesSymbol, make_namesvec());
   UNPROTECT(1);
   return res;
